searchData={"items":[{"type":"module","title":"Turboprop","doc":"","ref":"Turboprop.html"},{"type":"module","title":"Installation - Turboprop","doc":"Simply add the `turboprop` library to your `mix.exs`:\n\n`{:turboprop, \"~> 0.2\"}`\n\nIn addition, since Turboprop heavily utilises caching, its cache needs to be added to the supervision tree. In a default Phoenix\napplication, this is inside `application.ex`.\n\n```elixir\nchildren = [\n  # ...\n  Turboprop.Cache\n  # ...\n]\n```","ref":"Turboprop.html#module-installation"},{"type":"module","title":"Tools - Turboprop","doc":"Turboprop consists of multiple tools, each with their own purpose in building your component library.","ref":"Turboprop.html#module-tools"},{"type":"module","title":"Turboprop Hooks - Turboprop","doc":"Turboprop Hooks allow adding a ton of accessibility features to your components by simply adding a hook and a few data attributes to them.  \nThis includes:\n\n- Keyboard interactions\n- Focus management\n- ARIA attributes\n\nYou can either install and use them through the hex.pm dependency and some helpers we offer to add the relevant attributes to a component,\nor install them directly through npm and adding the attributes yourself.  \n\nAs an example, this renders a fully accessible dropdown menu:\n\n```heex\n \n   \n    Menu\n   \n   \n     \n       \n        Link\n       \n       Anchor \n     \n   \n \n```","ref":"Turboprop.html#module-turboprop-hooks"},{"type":"module","title":"Turboprop Merge - Turboprop","doc":"Turboprop Merge allows you to easily merge a list of Tailwind Classes to avoid style conflicts.\n\nImagine this component:\n\n```elixir\nattr :class, :string, doc: \"Class override\"\ndef button(assigns) do\n  ~H\"\"\"\n   Click me! \n  \"\"\"\nend\n```\n\nAnd imagine wanting to make the text a little bigger as a one-off. You've already added a `@class` attribute, but rendering the component\nwith `class=\"text-lg\"` will lead to an HTML output of `\"bg-black px-3 py-1.5 text-sm text-lg\"`, with two competing font size classes.\n\nNow, replace the `class` attribute with `class={merge([\"bg-black px-3 py-1.5 text-sm\", @class])}` and you will magically get\n`\"bg-black px-3 py-1.5 text-lg\"`.","ref":"Turboprop.html#module-turboprop-merge"},{"type":"module","title":"Turboprop Variants - Turboprop","doc":"Turboprop Variants is a variant API for TailwindCSS and Phoenix.\n\n```elixir\ndef button() do\n  %{\n    base: \"inline-flex items-center justify-center whitespace-nowrap rounded-md text-sm font-medium transition-colors focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:pointer-events-none disabled:opacity-50\",\n    variants: %{\n      variant: %{\n        default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n        destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n        outline: \"border border-input bg-background shadow-sm hover:bg-accent hover:text-accent-foreground\",\n        secondary: \"bg-secondary text-secondary-foreground shadow-sm hover:bg-secondary/80\",\n        ghost: \"hover:bg-accent hover:text-accent-foreground\",\n        link: \"text-primary underline-offset-4 hover:underline\"\n      },\n      size: %{\n        default: \"h-9 px-4 py-2\",\n        sm: \"h-8 rounded-md px-3 text-xs\",\n        lg: \"h-10 rounded-md px-8\",\n        icon: \"h-9 w-9\"\n      }\n    },\n    default_variants: [\n      variant: \"default\",\n      size: \"default\"\n    ]\n  }\nend\n\ndef render(assigns) do\n  ~H\"\"\"\n   A large, outlined button \n  \"\"\"\nend\n```\n\nIt supports variants, boolean variants, default variants, compound variants, slots, compound slots and more.","ref":"Turboprop.html#module-turboprop-variants"},{"type":"module","title":"Turboprop.Cache","doc":"","ref":"Turboprop.Cache.html"},{"type":"function","title":"Turboprop.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Turboprop.Cache.html#child_spec/1"},{"type":"function","title":"Turboprop.Cache.create_table/1","doc":"","ref":"Turboprop.Cache.html#create_table/1"},{"type":"function","title":"Turboprop.Cache.insert/3","doc":"","ref":"Turboprop.Cache.html#insert/3"},{"type":"function","title":"Turboprop.Cache.purge/1","doc":"","ref":"Turboprop.Cache.html#purge/1"},{"type":"function","title":"Turboprop.Cache.retrieve/2","doc":"","ref":"Turboprop.Cache.html#retrieve/2"},{"type":"function","title":"Turboprop.Cache.start_link/1","doc":"","ref":"Turboprop.Cache.html#start_link/1"},{"type":"module","title":"Turboprop.Hooks","doc":"Turboprop Hooks offer an easy way to build accessible components.  \n\nYou can easily add features like keyboard interaction, focus management and ARIA attributes to your component.  \nTurboprop Hooks are built around the state machines of [Zag](https://zagjs.com) and offer the same rules as them. You will find a link to\neach state machine's documentation in the hook's module.","ref":"Turboprop.Hooks.html"},{"type":"module","title":"Installation - Turboprop.Hooks","doc":"","ref":"Turboprop.Hooks.html#module-installation"},{"type":"module","title":"With mix - Turboprop.Hooks","doc":"If you use the esbuild setup Phoenix ships with by default, the hooks will automatically be available to your assets after adding\nTurboprop to your `mix.exs`. You then only need to add the hooks to your LiveView configuration.","ref":"Turboprop.Hooks.html#module-with-mix"},{"type":"module","title":"With npm - Turboprop.Hooks","doc":"The hooks are also available packaged as an npm package, [`@leuchtturm/turboprop`](https://npmjs.com/package/@leuchtturm/turboprop). Add \nit to your assets by running `npm i --save-exact @leuchtturm/turboprop` or the equivalent for the package manager you are using.","ref":"Turboprop.Hooks.html#module-with-npm"},{"type":"module","title":"Loading the hooks - Turboprop.Hooks","doc":"To reduce bundle size, it's recommended to only add the hooks actually used. For example, if only the `PinInput` hook is used:\n\n```javascript\nimport { PinInput } from \"@leuchtturm/turboprop\";\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: {\n    PinInput\n  },\n});\n```\n\nFor convenience, Turboprop also allows importing and adding all hooks at once:\n\n```javascript\nimport { Hooks } from \"@leuchtturm/turboprop\";\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: {\n    ...Hooks\n  },\n});\n```\n\nThat's it!","ref":"Turboprop.Hooks.html#module-loading-the-hooks"},{"type":"module","title":"Anatomy - Turboprop.Hooks","doc":"Each hook comes with a wrapper that adds the `phx-hook` directive and a bunch of `data` attributes that specify which role each element\nhas. Revisiting the dropdown menu example from the [Turboprop](`m:Turboprop`) documentation, the expanded elements look like this:\n\n```html\n \n   \n    Menu\n   \n   \n     \n       \n        Link\n       \n       \n        Anchor\n       \n     \n   \n \n```\nFor the `Menu` hook, the required parts are `trigger`, `positioner`, `content`, and `item`. In addition to that, it can also be configured\nwith optional parts like `separator` and `group`. Each hook's module will list both required and optional fields and how they relate to\neach other with nesting.  \nMost hooks only need the `data-part` attributes set, but some also have callbacks to the server where an event is required. Check the\ndocumentation for each hook.","ref":"Turboprop.Hooks.html#module-anatomy"},{"type":"function","title":"Turboprop.Hooks.id/0","doc":"","ref":"Turboprop.Hooks.html#id/0"},{"type":"module","title":"Turboprop.Hooks.Accordion","doc":"An accordion.","ref":"Turboprop.Hooks.Accordion.html"},{"type":"module","title":"Elements - Turboprop.Hooks.Accordion","doc":"","ref":"Turboprop.Hooks.Accordion.html#module-elements"},{"type":"module","title":"Required elements - Turboprop.Hooks.Accordion","doc":"- Root: Wrapper element.\n  - Required attributes:\n    - `data-part=\"root\"`\n- Item: Represents a single accordion item.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"item\"`\n    - `data-value` - A unique identifier for the item.\n- Item Trigger: Trigger that opens its item.\n  - Needs to be a child of an Item.\n  - Required attributes:\n    - `data-part=\"item-trigger\"`\n- Item Content: Content of an item.\n  - Needs to be a child of an Item.\n  - Required attributes:\n    - `data-part=\"item-content\"`","ref":"Turboprop.Hooks.Accordion.html#module-required-elements"},{"type":"module","title":"Optional elements - Turboprop.Hooks.Accordion","doc":"- Item Indicator: An indicator showing whether an item is opened.\n  - Needs to be a child of an Item.\n  - Required attributes:\n    - `data-part=\"item-indicator\"`","ref":"Turboprop.Hooks.Accordion.html#module-optional-elements"},{"type":"module","title":"Options - Turboprop.Hooks.Accordion","doc":"Options are set on the outer wrapper element through data attributes.  \nFor boolean attributes, adding the attribute with either an empty value or `\"true\"` is considered truthy, every other value falsy.\n\n- `data-disabled` - *boolean*: Whether the accordion is disabled and can be interacted with.\n- `data-multiple` - *boolean*: Whether multiple items can be opened at the same time.\n- `data-collapsible` - *boolean*: Whether an item can be closed by the user by clicking on it again. Implicitly set when `data-multiple`\nis set.","ref":"Turboprop.Hooks.Accordion.html#module-options"},{"type":"module","title":"Events - Turboprop.Hooks.Accordion","doc":"","ref":"Turboprop.Hooks.Accordion.html#module-events"},{"type":"module","title":"From the client - Turboprop.Hooks.Accordion","doc":"If set, the client will push the following events to the server:\n\n- `data-on-value-change`: Emitted when items are opened or closed.\n  - Sends an event with the type `%{value: list(binary())}` where the list contains the `data-value` of each opened item.","ref":"Turboprop.Hooks.Accordion.html#module-from-the-client"},{"type":"module","title":"Example - Turboprop.Hooks.Accordion","doc":"```heex\n \n   \n     \n       This is one element. \n       It's really difficult to come up with texts for these, to be honest. \n     \n     \n       This is another element. \n       Really, a second one isn't easier. \n     \n   \n \n```","ref":"Turboprop.Hooks.Accordion.html#module-example"},{"type":"module","title":"Turboprop.Hooks.Combobox","doc":"A combobox hook that provides an interactive dropdown list for selecting values.\n\nAllows users to either select an item from a predefined list or enter a custom value depending on configuration.","ref":"Turboprop.Hooks.Combobox.html"},{"type":"module","title":"Elements - Turboprop.Hooks.Combobox","doc":"","ref":"Turboprop.Hooks.Combobox.html#module-elements"},{"type":"module","title":"Required elements - Turboprop.Hooks.Combobox","doc":"- Root: Wrapper element.\n  - Required attributes:\n    - `data-part=\"root\"`\n- Control: Wrapper that holds the input and the trigger for the dropdown.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"control\"`\n- Input: Text input field where users search in the list.\n  - Needs to be a child of the Control.\n  - Required attributes:\n    - `data-part=\"input\"`\n- Trigger: Trigger that opens the menu.\n  - Needs to be a child of the Control.\n  - Required attributes:\n    - `data-part=\"trigger\"`\n- Positioner: A container for the dropdown content.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"positioner\"`\n- Content: The actual list of items.\n  - Needs to be a child of the Positioner.\n  - Required attributes:\n    - `data-part=\"content\"`\n- Item: Each selectable item in the dropdown list.\n  - Needs to be a child of the Positioner.\n  - Required attributes:\n    - `data-part=\"item\"`\n    - `data-value` - Value of the item.\n    - `data-label` - Human-readable label of the item.\n  - Styling:\n    - Receives a `data-highlighted` attribute when highlighted.\n    - Receives a `data-state=\"checked\"` attribute when selected.","ref":"Turboprop.Hooks.Combobox.html#module-required-elements"},{"type":"module","title":"Optional elements - Turboprop.Hooks.Combobox","doc":"- Label: A label for the combobox.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"label\"`","ref":"Turboprop.Hooks.Combobox.html#module-optional-elements"},{"type":"module","title":"Options - Turboprop.Hooks.Combobox","doc":"Options are set on the outer wrapper element through data attributes.\n\n- `data-name`: The name of the `input` field. Required for use in forms.\n- `data-input-behavior`: Determines how the input behaves in terms of autocomplete and highlighting.\n  - One of `autohighlight`, `autocomplete` or `none`.\n- `data-selection-behavior`: Determines how the selection behaves upon item selection.\n  - One of `clear`, `replace` or `preserve`.\n- `data-multiple` - *boolean*: Whether multiple items can be selected.\n- `data-loop-focus` - *boolean*: Whether focus should loop back to the top when reaching the end or vice versa.\n- `data-allow-custom-value` - *boolean*: Allows entering a value that is not in the predefined list.\n- `data-disabled` - *boolean*: Disables the combobox interaction.\n- `data-read-only` - *boolean*: Makes the combobox read-only.","ref":"Turboprop.Hooks.Combobox.html#module-options"},{"type":"module","title":"Events - Turboprop.Hooks.Combobox","doc":"","ref":"Turboprop.Hooks.Combobox.html#module-events"},{"type":"module","title":"From the client - Turboprop.Hooks.Combobox","doc":"- `data-on-open-change`: Emitted when the dropdown opens or closes.\n  - Sends an event with the type `%{open: boolean()}`.\n- `data-on-input-value-change`: Emitted when the input value changes.\n  - Sends an event with the type `%{inputValue: binary()}`.\n- `data-on-highlight-change`: Emitted when the highlighted item changes.\n  - Sends an event with the type `%{highlightedValue: binary(), highlightedItem: %{value: binary(), label: binary()}}`.\n- `data-on-value-change`: Emitted when the selected value changes.\n  - Sends an event with the type `%{value: list(binary()), items: list(%{value: binary(), label: binary()})}`.","ref":"Turboprop.Hooks.Combobox.html#module-from-the-client"},{"type":"module","title":"Example - Turboprop.Hooks.Combobox","doc":"```elixir\n@items [\n  %{label: \"United States\", id: \"US\"},\n  %{label: \"Germany\", id: \"DE\"},\n  %{label: \"India\", id: \"IN\"},\n  %{label: \"Brazil\", id: \"BR\"},\n  %{label: \"Canada\", id: \"CA\"},\n  %{label: \"France\", id: \"FR\"},\n  %{label: \"Japan\", id: \"JP\"},\n  %{label: \"South Korea\", id: \"KR\"},\n  %{label: \"Australia\", id: \"AU\"},\n  %{label: \"Italy\", id: \"IT\"},\n  %{label: \"Spain\", id: \"ES\"},\n  %{label: \"Nigeria\", id: \"NG\"},\n  %{label: \"Russia\", id: \"RU\"},\n  %{label: \"China\", id: \"CN\"},\n  %{label: \"United Kingdom\", id: \"GB\"},\n  %{label: \"Mexico\", id: \"MX\"},\n  %{label: \"Indonesia\", id: \"ID\"},\n  %{label: \"Turkey\", id: \"TR\"},\n  %{label: \"South Africa\", id: \"ZA\"},\n  %{label: \"Sweden\", id: \"SE\"},\n  %{label: \"Argentina\", id: \"AR\"},\n  %{label: \"Belgium\", id: \"BE\"},\n  %{label: \"Thailand\", id: \"TH\"},\n  %{label: \"New Zealand\", id: \"NZ\"},\n  %{label: \"Norway\", id: \"NO\"}\n]\n\ndef mount(_params, _session, socket) do\n  {:ok, assign(socket, :items, @items)}\nend\n\ndef handle_event(\"search\", %{\"inputValue\" => query}, socket) do\n  items = if query == \"\", do: @items, else: Enum.filter(@items, &String.starts_with?(String.downcase(&1.label), String.downcase(query)))\n  {:noreply, assign(socket, :items, items)}\nend\n\ndef handle_event(\"save\", %{\"value\" => value}, socket) do\n  # Do something with value...\n\n  {:noreply, socket}\nend\n\ndef render(assigns) do\n  ~H\"\"\"\n   \n     Select country \n     \n       \n         \n         Open \n       \n     \n     \n       \n         \n          <%= item.label %>\n         \n       \n     \n   \n  \"\"\"\nend\n```","ref":"Turboprop.Hooks.Combobox.html#module-example"},{"type":"module","title":"Turboprop.Hooks.Dialog","doc":"A dialog component, also known as Modal.","ref":"Turboprop.Hooks.Dialog.html"},{"type":"module","title":"Elements - Turboprop.Hooks.Dialog","doc":"","ref":"Turboprop.Hooks.Dialog.html#module-elements"},{"type":"module","title":"Required elements - Turboprop.Hooks.Dialog","doc":"- Trigger: Trigger that opens the dialog.\n  - Required attributes:\n    - `data-part=\"trigger\"`\n- Positioner: Controls the dialog's position in the viewport.\n  - Required attributes:\n    - `data-part=\"positioner\"`\n- Content: Contains the dialog content.\n  - Needs to be a child of the Positioner.\n  - Required attributes:\n    - `data-part=\"content\"`","ref":"Turboprop.Hooks.Dialog.html#module-required-elements"},{"type":"module","title":"Optional elements - Turboprop.Hooks.Dialog","doc":"- Backdrop: Backdrop to cover the rest of the viewport.\n  - Required attributes:\n    - `data-part=\"backdrop\"`\n- Title: Title of the dialog.\n  - Needs to be a child of the Content.\n  - Required attributes:\n    - `data-part=\"title\"`\n- Description: Description of the dialog.\n  - Needs to be a child of the Content.\n  - Required attributes:\n    - `data-part=\"description\"`\n- Close trigger: Trigger to close the dialog.\n  - Required attributes:\n    - `data-part=\"close-trigger\"`","ref":"Turboprop.Hooks.Dialog.html#module-optional-elements"},{"type":"module","title":"Options - Turboprop.Hooks.Dialog","doc":"Options are set on the outer wrapper element through data attributes.  \nFor boolean attributes, adding the attribute with either an empty value or `\"true\"` is considered truthy, every other value falsy.\n\n- `data-role`: Role of the dialog.\n  - One of `dialog` or `alertdialog`.\n- `data-prevent-scroll` - *boolean*: Prevent scrolling of the body behind the dialog. Defaults to `true`.\n- `data-close-on-interact-outside` - *boolean*: Close the dialog when interacting with an element outside of it. Defaults to `true`.\n- `data-close-on-escape` - *boolean*: Close the dialog when the Escape key is pressed. Defaults to `true`.","ref":"Turboprop.Hooks.Dialog.html#module-options"},{"type":"module","title":"Events - Turboprop.Hooks.Dialog","doc":"","ref":"Turboprop.Hooks.Dialog.html#module-events"},{"type":"module","title":"From the client - Turboprop.Hooks.Dialog","doc":"If set, the client will push the following events to the server:\n\n- `data-on-open-change`: Emitted when the open state of the dialog changes.\n  - Sends an event with the type `%{open: boolean()}`","ref":"Turboprop.Hooks.Dialog.html#module-from-the-client"},{"type":"module","title":"Example - Turboprop.Hooks.Dialog","doc":"```heex\n \n   \n    Open dialog\n   \n    \n   \n     \n       Dialog \n       Welcome to Turboprop! \n       Close \n     \n   \n \n```","ref":"Turboprop.Hooks.Dialog.html#module-example"},{"type":"module","title":"Turboprop.Hooks.Menu","doc":"A dropdown menu.","ref":"Turboprop.Hooks.Menu.html"},{"type":"module","title":"Elements - Turboprop.Hooks.Menu","doc":"","ref":"Turboprop.Hooks.Menu.html#module-elements"},{"type":"module","title":"Required elements - Turboprop.Hooks.Menu","doc":"- Trigger: Trigger that opens the menu.\n  - Required attributes:\n    - `data-part=\"trigger\"`\n- Positioner: Controls the menu's position in the viewport.\n  - Required attributes:\n    - `data-part=\"positioner\"`\n- Content: Contains the menu content.\n  - Needs to be a child of the Positioner.\n  - Required attributes:\n    - `data-part=\"content\"`\n- Item: Represents an individual action or option within the menu.\n  - Needs to be a child of the Content or optionally an Item Group.\n  - Required attributes:\n    - `data-part=\"item\"`\n    - `data-value` - A unique identifier for the item.\n  - Styling:\n    - Receives a `data-highlighted` attribute when highlighted.","ref":"Turboprop.Hooks.Menu.html#module-required-elements"},{"type":"module","title":"Optional elements - Turboprop.Hooks.Menu","doc":"- Item Group: A group of Items.\n  - Needs to be a child of the Content.\n  - Required attributes:\n    - `data-part=\"item-group\"`\n    - `data-value` - A unique identifier for the item group.\n    - `name` - A unique name for the item group, if labelled.\n- Item Group Label: A label for an Item Group.\n  - Needs to be a child of the Content.\n  - Required attributes:\n    - `data-part=\"item-group-label\"`\n    - `for` - The `name` attribute of the Item Group the label belongs to.\n- Separator: A separator between Items.\n  - Needs to be a child of the Content.\n  - Required attributes:\n    - `data-part=\"separator\"`","ref":"Turboprop.Hooks.Menu.html#module-optional-elements"},{"type":"module","title":"Example - Turboprop.Hooks.Menu","doc":"```heex\n \n   Menu \n   \n     \n      <.link navigate=\"/dashboard\" class=\"data-[highlighted]:bg-gray-100\" data-part=\"item\" data-value=\"dashboard\">\n        Dashboard\n      </.link>\n       \n       Account \n       \n        <.link navigate=\"/signup\" data-part=\"item\" data-value=\"signup\">Sign up</.link>\n        <.link navigate=\"/login\" data-part=\"item\" data-value=\"login\">Login</.link>\n       \n     \n   \n \n```","ref":"Turboprop.Hooks.Menu.html#module-example"},{"type":"module","title":"Turboprop.Hooks.PinInput","doc":"Input for PINs and OTPs.\n\nConsists of multiple inputs, each allowing one character at a time. When the digit or letter is entered, focus transfers to the next \ninput in the sequence, until every input is filled.","ref":"Turboprop.Hooks.PinInput.html"},{"type":"module","title":"Elements - Turboprop.Hooks.PinInput","doc":"","ref":"Turboprop.Hooks.PinInput.html#module-elements"},{"type":"module","title":"Required elements - Turboprop.Hooks.PinInput","doc":"- Root: Wrapper element.\n  - Required attributes:\n    - `data-part=\"root\"`\n- Input: An input for a single character.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"input\"`\n    - `data-index` - An integer describing the index of the current input. Needs to start at `0` for the first input.","ref":"Turboprop.Hooks.PinInput.html#module-required-elements"},{"type":"module","title":"Optional elements - Turboprop.Hooks.PinInput","doc":"- Label: A label for the pin input.\n  - Needs to be a child of the Root.\n  - Required attributes:\n    - `data-part=\"label\"`","ref":"Turboprop.Hooks.PinInput.html#module-optional-elements"},{"type":"module","title":"Options - Turboprop.Hooks.PinInput","doc":"Options are set on the outer wrapper element through data attributes.  \nFor boolean attributes, adding the attribute with either an empty value or `\"true\"` is considered truthy, every other value falsy.\n\n- `data-type`: The type of the input. Defines which characters are allowed.\n  - One of `alphanumeric`, `numeric` or `alphabetic`.\n- `data-otp` - *boolean*: Add when the input is a One-Time Password to correctly set the `autocomplete` tag.\n- `data-blur-on-complete` - *boolean*: Blur the last input when the user completes the pin input.\n- `data-placeholder`: The placeholder to show in each input field.\n- `data-dir`: Set to `rtl` if the pin should be entered right-to-left.\n  - One of `ltr` or `rtl`.","ref":"Turboprop.Hooks.PinInput.html#module-options"},{"type":"module","title":"Events - Turboprop.Hooks.PinInput","doc":"","ref":"Turboprop.Hooks.PinInput.html#module-events"},{"type":"module","title":"From the client - Turboprop.Hooks.PinInput","doc":"If set, the client will push the following events to the server:\n\n- `data-on-change`: Emitted when any of the inputs change.\n  - Sends an event with the type `%{value: list(binary()), valueAsString: binary()}`\n- `data-on-complete`: Emitted when all inputs have been filled.\n  - Sends an event with the type `%{value: list(binary()), valueAsString: binary()}`\n- `data-on-invalid`: Emitted when the entire input is invalid.\n  - Sends an event with the type `%{value: binary(), index: integer()}`","ref":"Turboprop.Hooks.PinInput.html#module-from-the-client"},{"type":"module","title":"From the server - Turboprop.Hooks.PinInput","doc":"The following events can be sent to the component through `push_event/3`.\n\n- `clear`: Clears the input. Accepts no parameters.","ref":"Turboprop.Hooks.PinInput.html#module-from-the-server"},{"type":"module","title":"Example - Turboprop.Hooks.PinInput","doc":"```elixir\ndef render(assigns) do\n  ~H\"\"\"\n   \n     \n       OTP \n       \n         \n         \n         \n         \n         \n         \n       \n     \n   \n  \"\"\"\nend\n\n@impl true\ndef handle_event(\"submit\", %{\"valueAsString\" => value}, socket) do\n  # Do something with value...\n  {:noreply, socket}\n  \nend\n```","ref":"Turboprop.Hooks.PinInput.html#module-example"},{"type":"module","title":"Turboprop.Merge","doc":"Turboprop Merge adds efficient class joining and merging of TailwindCSS classes to Elixir.\n\nTailwindCSS class names are composable and allow specifying an infinite amount of different styles. Most components allow overriding class\nnames, like as passing `class` attribute that then gets merged with the existing styles. This can result in class lists such as\n`text-white bg-red-500 bg-blue-300` where `text-white bg-red-500` is the preset style, and `bg-blue-300` is the override for that one\nspecific button that needs to look slightly different.  \nStyles based on class are applied according to the order _they are defined at in the stylesheet_. In this example, because TailwindCSS\norders color definitions alphabetically, the override does not work. `blue` is defined before `red`, so the `bg-red-500` class takes\nprecedence since it was defined later.  \n\nIn order to still allow overriding styles, Turboprop Merge traverses the entire class list, creates a list of all classes and which\nconflicting groups of styles exist in them and gives precedence to the ones that were defined last _in the class list_, which, unlike the\nstylesheet, is in control of the user.","ref":"Turboprop.Merge.html"},{"type":"module","title":"Example - Turboprop.Merge","doc":"```elixir\niex> merge(\"text-white bg-red-500 bg-blue-300\")\n\"text-white bg-blue-300\"\n\niex> merge([\"px-2 py-1 bg-red hover:bg-dark-red\", \"p-3 bg-[#B91C1C]\"])\n\"hover:bg-dark-red p-3 bg-[#B91C1C]\"\n```","ref":"Turboprop.Merge.html#module-example"},{"type":"module","title":"Configuration - Turboprop.Merge","doc":"Turboprop Merge does not currently support full theme configuration - that's on the roadmap!  \n\nThe limited configuration at the moment is adding Tailwind's `prefix` option.\n\n```elixir\nconfig :turboprop,\n  prefix: \"tw-\"\n```","ref":"Turboprop.Merge.html#module-configuration"},{"type":"function","title":"Turboprop.Merge.join/1","doc":"Joins a list of classes.","ref":"Turboprop.Merge.html#join/1"},{"type":"function","title":"Turboprop.Merge.merge/2","doc":"Joins and merges a list of classes.\n\nPasses the input to `join/1` before merging.","ref":"Turboprop.Merge.html#merge/2"},{"type":"module","title":"Turboprop.Variants","doc":"Turboprop Variants adds a feature-rich variant API for TailwindCSS to Elixir.\n\nVariants allow you to define all possible styles and options for a component, even one that spans multiple HTML elements, in one place,\nand then apply them based on your assigns.","ref":"Turboprop.Variants.html"},{"type":"module","title":"Example - Turboprop.Variants","doc":"```elixir\ndef button() do\n  [\n    base: \"inline-flex items-center justify-center whitespace-nowrap rounded-md text-sm font-medium transition-colors focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:pointer-events-none disabled:opacity-50\",\n    variants: [\n      variant: [\n        default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n        destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n        outline: \"border border-input bg-background shadow-sm hover:bg-accent hover:text-accent-foreground\",\n        secondary: \"bg-secondary text-secondary-foreground shadow-sm hover:bg-secondary/80\",\n        ghost: \"hover:bg-accent hover:text-accent-foreground\",\n        link: \"text-primary underline-offset-4 hover:underline\"\n      ],\n      size: [\n        default: \"h-9 px-4 py-2\",\n        sm: \"h-8 rounded-md px-3 text-xs\",\n        lg: \"h-10 rounded-md px-8\",\n        icon: \"h-9 w-9\"\n      ]\n    ],\n    default_variants: [\n      variant: \"default\",\n      size: \"default\"\n    ]\n  }\nend\n```","ref":"Turboprop.Variants.html#module-example"},{"type":"module","title":"Features - Turboprop.Variants","doc":"Turboprop Variants comes with a ton of features to manage your variants.","ref":"Turboprop.Variants.html#module-features"},{"type":"module","title":"Variants - Turboprop.Variants","doc":"You can add variants and their options inside the `variants` map. \n\n```elixir\niex> alert = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-background text-foreground\",\n...>       destructive: \"border-destructive/50 text-destructive dark:border-destructive [&>svg]:text-destructive\",\n...>     ]\n...>   ]\n...> ]\niex> variant(alert, variant: \"destructive\")\n\"border-destructive/50 text-destructive dark:border-destructive [&>svg]:text-destructive\"\n```\n\n#### Multiple variants\n\nEach component can have any number of variants.\n\n```elixir\niex> button = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n...>       destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n...>       outline: \"border border-input bg-background shadow-sm hover:bg-accent hover:text-accent-foreground\",\n...>       secondary: \"bg-secondary text-secondary-foreground shadow-sm hover:bg-secondary/80\",\n...>       ghost: \"hover:bg-accent hover:text-accent-foreground\",\n...>       link: \"text-primary underline-offset-4 hover:underline\",\n...>     ],\n...>     size: [\n...>       default: \"h-9 px-4 py-2\",\n...>       sm: \"h-8 rounded-md px-3 text-xs\",\n...>       lg: \"h-10 rounded-md px-8\",\n...>       icon: \"h-9 w-9\",\n...>     ],\n...>   ],\n...> ]\niex> variant(button, variant: \"destructive\", size: \"sm\")\n\"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90 h-8 rounded-md px-3 text-xs\"\n```\n\n#### Boolean variants\n\nSome components benefit from having boolean variants, such as `disabled`.  \n\nPassing only one of `true` and `false` as options is allowed. By default, if the variant is not passed as a keyword and if it exists, the\n`false` value is applied.\n\n```elixir\niex> button = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n...>       destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n...>     ],\n...>     disabled: [\n...>       true: \"opacity-50 bg-gray-500 hover:bg-gray-500 pointer-events-none\"\n...>     ],\n...>   ],\n...> ]\niex> variant(button, variant: \"default\")\n\"bg-primary text-primary-foreground shadow hover:bg-primary/90\"\niex> variant(button, variant: \"destructive\", disabled: true)\n\"text-destructive-foreground shadow-sm opacity-50 bg-gray-500 hover:bg-gray-500 pointer-events-none\"\n```\n\n#### Default variants\n\nDefault variants can easily be set so they do not need to be passed every time.\n\n```elixir\niex> button = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n...>       destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n...>       outline: \"border border-input bg-background shadow-sm hover:bg-accent hover:text-accent-foreground\",\n...>       secondary: \"bg-secondary text-secondary-foreground shadow-sm hover:bg-secondary/80\",\n...>       ghost: \"hover:bg-accent hover:text-accent-foreground\",\n...>       link: \"text-primary underline-offset-4 hover:underline\",\n...>     ],\n...>     size: [\n...>       default: \"h-9 px-4 py-2\",\n...>       sm: \"h-8 rounded-md px-3 text-xs\",\n...>       lg: \"h-10 rounded-md px-8\",\n...>       icon: \"h-9 w-9\",\n...>     ],\n...>   ],\n...>   default_variants: [\n...>     # Both strings and atoms are fine!\n...>     variant: \"default\",\n...>     size: :default\n...>   ]\n...> ]\niex> variant(button)\n\"bg-primary text-primary-foreground shadow hover:bg-primary/90 h-9 px-4 py-2\"\n```\n\n#### Compound variants\n\nTurboprop Variants supports variants that depend on other variants.\n\n```elixir\niex> button = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-primary text-primary-foreground shadow hover:bg-primary/90\",\n...>       destructive: \"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90\",\n...>     ],\n...>     disabled: [\n...>       true: \"opacity-50 bg-gray-500 pointer-events-none\"\n...>     ],\n...>   ],\n...>   compound_variants: [\n...>     [\n...>       variant: \"destructive\",\n...>       disabled: false,\n...>       class: \"focus:ring-1\"\n...>     ]\n...>   ]\n...> ]\niex> variant(button, variant: \"destructive\", disabled: false)\n\"bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90 focus:ring-1\"\n```\n\nThe variants also accept arrays, in which case the compound variant will be applied when any of the values match.","ref":"Turboprop.Variants.html#module-variants"},{"type":"module","title":"Slots - Turboprop.Variants","doc":"Slots allow spreading a component's variants over multiple elements.\n\nWhen no `slot` option is provided, the `base` one is implied.  \nTo be consistent with when not using slots, the `base` slot can also be defined _outside_ the `slots` map.\n\n```elixir\niex> card = [\n...>   # Can also go here:\n...>   # base: \"md:flex bg-slate-100 rounded-xl p-8 md:p-0 dark:bg-gray-900\",\n...>   slots: [\n...>     base: \"md:flex bg-slate-100 rounded-xl p-8 md:p-0 dark:bg-gray-900\",\n...>     avatar: \"w-24 h-24 md:w-48 md:h-auto md:rounded-none rounded-full mx-auto drop-shadow-lg\",\n...>     wrapper: \"flex-1 pt-6 md:p-8 text-center md:text-left space-y-4\",\n...>     description: \"text-md font-medium\",\n...>   ]\n...> ]\niex> variant(card)\n\"md:flex bg-slate-100 rounded-xl p-8 md:p-0 dark:bg-gray-900\"\niex> variant(card, slot: :wrapper)\n\"flex-1 pt-6 md:p-8 text-center md:text-left space-y-4\"\n```\n\n#### Slots with variants\n\nSlots seamlessly work together with variants.\n\n```elixir\niex> card = [\n...>   slots: [\n...>     base: \"md:flex rounded-xl p-8 md:p-0\",\n...>     avatar: \"md:h-auto md:rounded-none rounded-full mx-auto drop-shadow-lg\",\n...>   ],\n...>   variants: [\n...>     color: [\n...>       gray: [\n...>         base: \"bg-slate-100 dark:bg-gray-900\"\n...>       ],\n...>       red: [\n...>         base: \"bg-red-100 dark:bg-red-900\"\n...>       ]\n...>     ],\n...>     size: [\n...>       sm: [\n...>         avatar: \"w-24 h-24\"\n...>       ],\n...>       lg: [\n...>         avatar: \"w-48 h-48\"\n...>       ]\n...>     ]\n...>   ]\n...> ]\niex> variant(card, color: \"gray\")\n\"md:flex rounded-xl p-8 md:p-0 bg-slate-100 dark:bg-gray-900\"\niex> variant(card, color: \"red\")\n\"md:flex rounded-xl p-8 md:p-0 bg-red-100 dark:bg-red-900\"\niex> variant(card, slot: :avatar, size: \"lg\")\n\"md:h-auto md:rounded-none rounded-full mx-auto drop-shadow-lg w-48 h-48\"\n```\n\nCompound variants can be similarly used with slots.\n\n#### Compound slots\n\nLike compound variants, Turboprop Variants also supports adding clases depending on which variants are applied.  \nThis is the \"Everything everywhere all at once\" usage.\n\n```elixir\niex> pagination = [\n...>   slots: [\n...>     base: \"flex flex-wrap relative gap-1 max-w-fit\",\n...>     item: \"data-[active='true']:bg-blue-500 data-[active='true']:text-white\",\n...>   ],\n...>   variants: [],\n...>   default_variants: [\n...>     size: \"md\"\n...>   ],\n...>   compound_slots: [\n...>     [\n...>       slots: [:item, :prev, :next],\n...>       class: [\n...>         \"flex\",\n...>         \"flex-wrap\",\n...>         \"truncate\",\n...>         \"box-border\",\n...>         \"outline-none\",\n...>         \"items-center\",\n...>         \"justify-center\",\n...>         \"bg-neutral-100\",\n...>         \"hover:bg-neutral-200\",\n...>         \"active:bg-neutral-300\",\n...>         \"text-neutral-500\"\n...>       ]\n...>     ],\n...>     [\n...>       slots: [:item, :prev, :next],\n...>       size: \"xs\",\n...>       class: \"w-7 h-7 text-xs\"\n...>     ],\n...>     [\n...>       slots: [:item, :prev, :next],\n...>       size: \"sm\",\n...>       class: \"w-8 h-8 text-sm\"\n...>     ],\n...>     [\n...>       slots: [:item, :prev, :next],\n...>       size: \"md\",\n...>       class: \"w-9 h-9 text-base\"\n...>     ]\n...>   ]\n...> ]\niex> variant(pagination)\n\"flex flex-wrap relative gap-1 max-w-fit\"\niex> variant(pagination, slot: :item)\n\"data-[active='true']:bg-blue-500 data-[active='true']:text-white flex flex-wrap truncate box-border outline-none items-center justify-center bg-neutral-100 hover:bg-neutral-200 active:bg-neutral-300 text-neutral-500 w-9 h-9 text-base\"\niex> variant(pagination, slot: :prev, size: \"xs\")\n\"flex flex-wrap truncate box-border outline-none items-center justify-center bg-neutral-100 hover:bg-neutral-200 active:bg-neutral-300 text-neutral-500 w-7 h-7 text-xs\"\n```","ref":"Turboprop.Variants.html#module-slots"},{"type":"module","title":"Overriding class names - Turboprop.Variants","doc":"For when that one special element is needed, class overrides can also be passed:\n\n\n```elixir\niex> alert = [\n...>   variants: [\n...>     variant: [\n...>       default: \"bg-background text-foreground\",\n...>       destructive: \"border-destructive/50 text-destructive dark:border-destructive [&>svg]:text-destructive\",\n...>     ]\n...>   ]\n...> ]\niex> variant(alert, variant: \"default\", class: \"bg-yellow-500\")\n\"text-foreground bg-yellow-500\"\n```","ref":"Turboprop.Variants.html#module-overriding-class-names"},{"type":"function","title":"Turboprop.Variants.variant/2","doc":"Computes a component's classes.","ref":"Turboprop.Variants.html#variant/2"},{"type":"function","title":"Parameters - Turboprop.Variants.variant/2","doc":"- `component`: A component definition. Refer to the module documentation for its options and structure.\n- `selectors`: A list of selectors.","ref":"Turboprop.Variants.html#variant/2-parameters"},{"type":"function","title":"Selectors - Turboprop.Variants.variant/2","doc":"Selectors are a keyword list containing a few different keys.\n\n- `slot`: If the passed component has defined slots, they can be selected by passing the `slot` selector. It has to be an atom and\ndefaults to `base` if not present.\n- Variants: Pass selectors for each variant. Values can be either atoms or strings.","ref":"Turboprop.Variants.html#variant/2-selectors"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"0.4.1 - 2024-06-19 - Changelog","doc":"","ref":"changelog.html#0-4-1-2024-06-19"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed npm config.","ref":"changelog.html#fixed"},{"type":"extras","title":"0.4.0 -- 2024-06-19 - Changelog","doc":"","ref":"changelog.html#0-4-0-2024-06-19"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Variants declarations are now using keyword maps everywhere instead of a mix of maps and keyword lists.\n- Hooks no longer come with helper functions in Elixir. These have been removed.\n- PinInput events now send the full details object given by Zag.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Added - Changelog","doc":"- New hook: `Turboprop.Hooks.Accordion`.\n- New hook: `Turboprop.Hooks.Combobox`.\n- Hooks are now written in TypeScript and are fully typed.\n- Hook attributes are now properly validated and any errors output in the browser console.\n- Hooks no longer come with helper functions in Elixir. These have been removed.\n- PinInput events now send the full details object given by Zag.\n- Hook examples no longer include styling.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Variants declarations are now using keyword maps everywhere instead of a mix of maps and keyword lists.","ref":"changelog.html#changed"},{"type":"extras","title":"0.3.1 -- 2024-06-16 - Changelog","doc":"","ref":"changelog.html#0-3-1-2024-06-16"},{"type":"extras","title":"Fixed - Changelog","doc":"- Add forgotten `start_link/1` method to cache.","ref":"changelog.html#fixed"},{"type":"extras","title":"0.3.0 -- 2024-06-15 - Changelog","doc":"","ref":"changelog.html#0-3-0-2024-06-15"},{"type":"extras","title":"Changed - Changelog","doc":"- Use ETS for caching. This requires adding `Turboprop.Cache` to the supervision tree.\n- Updated Zag","ref":"changelog.html#changed"},{"type":"extras","title":"0.2.1 -- 2024-06-14 - Changelog","doc":"","ref":"changelog.html#0-2-1-2024-06-14"},{"type":"extras","title":"Fixed - Changelog","doc":"- Do not raise when a non-existent variant option is passed as string.\n- Apply class overrides last","ref":"changelog.html#fixed"},{"type":"extras","title":"0.2.0 - 2024-06-12 - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-06-12"},{"type":"extras","title":"Added - Changelog","doc":"- New hook: `Turboprop.Hooks.Dialog`\n- New tool: `Turboprop.Variants`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated Zag","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.4 - 2024-06-11 - Changelog","doc":"","ref":"changelog.html#0-1-4-2024-06-11"},{"type":"extras","title":"Added - Changelog","doc":"- New hook: `Turboprop.Hooks.PinInput`","ref":"changelog.html#added"},{"type":"extras","title":"0.1.3 - 2024-06-10 - Changelog","doc":"Initial release.","ref":"changelog.html#0-1-3-2024-06-10"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}